from flask_jwt_extended import jwt_required
from flask_restplus import Namespace, Resource

from portal import logger
from portal.rest import rest_helpers
from portal.rest.schemas import SCHEMAS
from portal.service import general, permissions
from portal.service.workers import WorkerService

workers_namespace = Namespace('workers')
log = logger.get_logger(__name__)


@workers_namespace.route('')
class WorkerList(Resource):
    @jwt_required
    # @workers_namespace.response(200, 'List of workers', model=workers_schema)
    @workers_namespace.response(403, 'Not allowed to list workers')
    def get(self):
        permissions.PermissionsService().require.sysadmin()
        workers_list = WorkerService().find_all_workers()
        return SCHEMAS.dump('workers', workers_list)

    @jwt_required
    # @workers_namespace.response(201, 'Created worker', model=worker_schema)
    @workers_namespace.response(403, 'Not allowed to add worker')
    def post(self):
        permissions.PermissionsService().require.sysadmin()
        data = rest_helpers.parse_request_data(action='create', resource='worker')
        new_worker = WorkerService().create_worker(**data)
        return SCHEMAS.dump('worker', new_worker), 201


@workers_namespace.route('/<string:wid>')
@workers_namespace.param('wid', 'Worker id')
@workers_namespace.response(404, 'Worker not found')
class WorkerResource(Resource):

    @jwt_required
    # @workers_namespace.response(200, 'Worker', model=worker_schema)
    @workers_namespace.response(403, 'Not allowed to access worker')
    def get(self, wid: str):
        permissions.PermissionsService().require.sysadmin_or_self(wid)
        worker = general.find_worker(wid)
        return SCHEMAS.dump('worker', worker)

    @jwt_required
    @workers_namespace.response(204, 'Worker deleted')
    @workers_namespace.response(403, 'Not allowed to delete worker')
    def delete(self, wid: str):
        permissions.PermissionsService().require.sysadmin()
        worker = general.find_worker(wid)
        WorkerService(worker=worker).delete_worker()
        return '', 204

    @jwt_required
    @workers_namespace.response(204, 'Worker updated')
    @workers_namespace.response(403, 'Not allowed to update worker')
    def put(self, wid: str):
        permissions.PermissionsService().require.sysadmin_or_self(wid)

        data = rest_helpers.parse_request_data(action='update', resource='worker', partial=True)

        worker = general.find_worker(wid)
        WorkerService(worker=worker).update_worker(data)
        return '', 204